#pragma option -l;
#ifndef __16C64_H
#define __16C64_H
/*
        MPC Code Development System
        Header file for PIC16C64        V1.12C

                        16C64.H

        (c) Copyright 1995   Byte Craft Limited
        421 King St.N., Waterloo, ON, Canada, N2J 4E4

        Sherif Abdel-Kader

 SAK Fixed address of temp_longIX_H
*/

// Processor core
#pragma has PIC14;

// Procssor type
#pragma has PIC16C64;

#pragma option +l;

// Reset and Interrupt vectors
#pragma vector __RESET @ 0x0000;
#pragma vector __INT   @ 0x0004;

// ROM and RAM size/starting address
#define MAXROM  0x800   // Total program memory space ( 2K words)
#define MAXB0   0x80    // Register file Bank 0 space (128 words)
#define MAXB1   0xC0    // Register file Bank 1 space (128 words)
#pragma memory ROM [MAXROM - 0x05]  @ 0x05;
#ifdef NOLONG
#pragma memory RAM [MAXB0  - 0x27]  @ 0x27;
#else
#pragma memory RAM [MAXB0  - 0x2F]  @ 0x2F;
#endif
#pragma memory RAM [MAXB1  - 0xA1]  @ 0xA1;

char temp_WREG     @  0x20;   // Also reserve 0xA0 for it
char __WImage      @  0x21;
char temp_WImage   @  0x22;
char temp_STATUS   @  0x23;
char temp_FSR      @  0x24;
char __FSRImage    @  0x25;
char temp_FSRImage @  0x26;
#pragma option -l;

#ifndef NOLONG
#pragma option +l;
  long __longAC      @  0x27;
  char __longAC_L    @  0x27;
  char __longAC_H    @  0x28;
  long __longIX      @  0x29;
  char __longIX_L    @  0x29;
  char __longIX_H    @  0x2A;
  char temp_longAC_L @  0x2B;
  char temp_longAC_H @  0x2C;
  char temp_longIX_L @  0x2D;
  char temp_longIX_H @  0x2E;
#pragma option -l;
#endif

// Working (W) register
registerw  WREG;

// Special-purpose Registers, Bank 0
#pragma portrw INDF       @ 0x00;
#pragma portrw TMR0       @ 0x01;
#pragma portrw PCL        @ 0x02;
#pragma portrw STATUS     @ 0x03;
#pragma portrw FSR        @ 0x04;
#pragma portrw PORTA      @ 0x05;
#pragma portrw PORTB      @ 0x06;
#pragma portrw PORTC      @ 0x07;
#pragma portrw PORTD      @ 0x08;
#pragma portrw PORTE      @ 0x09;
#pragma portrw PCLATH     @ 0x0A;
#pragma portrw INTCON     @ 0x0B;
#pragma portrw PIR1       @ 0x0C;
#pragma portrw TMR1L      @ 0x0E;
#pragma portrw TMR1H      @ 0x0F;
#pragma portrw T1CON      @ 0x10;
#pragma portrw TMR2       @ 0x11;
#pragma portrw T2CON      @ 0x12;
#pragma portrw SSPBUF     @ 0x13;
#pragma portrw SSPCON     @ 0x14;
#pragma portrw CCPR1L     @ 0x15;
#pragma portrw CCPR1H     @ 0x16;
#pragma portrw CCP1CON    @ 0x17;

// Special-purpose Registers, Bank 1
#pragma portrw OPTION     @ 0x81;
#pragma portrw TRISA      @ 0x85;
#pragma portrw TRISB      @ 0x86;
#pragma portrw TRISC      @ 0x87;
#pragma portrw TRISD      @ 0x88;
#pragma portrw TRISE      @ 0x89;
#pragma portrw PIE1       @ 0x8C;
#pragma portrw PCON       @ 0x8E;
#pragma portrw PR2        @ 0x92;
#pragma portrw SSPADD     @ 0x93;
#pragma portrw SSPSTAT    @ 0x94;

// STATUS register bits
#define C       0
#define DC      1
#define Z       2
#define PD_     3
#define TO_     4
#define RP0     5
#define RP1     6
#define IRP     7

// PORTA register bits (only special pins)
#define T0CKI   4
#define SS_	5

// PORTB register bits (only special pins)
#define INT     0

// PORTC register bits (only special pins)
#define T1CKI   0
#define T1OSI   0
#define T1OSO   1
#define CCP1    2
#define SCK     3
#define SCL     3
#define SDI     4
#define SDA     4
#define SDO     5

// PORTE register bits (only special pins)
#define RD_     0
#define WR_     1
#define CS_     2

//INTCON register bits
#define RBIF    0
#define INTF    1
#define T0IF    2
#define RBIE    3
#define INTE    4
#define T0IE    5
#define PEIE    6
#define GIE     7

// PIR1 register bits
#define TMR1IF  0
#define TMR2IF  1
#define CCP1IF  2
#define SSPIF   3
#define PSPIF   7

// T1CON register bits
#define TMR1ON  0
#define TMR1CS  1
#define T1SYNC_ 2
#define T1OSCEN 3
#define T1CKPS0 4
#define T1CKPS1 5

// T2CON register bits
#define T2CKPS0 0
#define T2CKPS1 1
#define TMR2ON  2
#define TOUTPS0 3
#define TOUTPS1 4
#define TOUTPS2 5
#define TOUTPS3 6

// SSPCON register bits
#define SSPM0   0
#define SSPM1   1
#define SSPM2   2
#define SSPM3   3
#define CKP     4
#define SSPEN   5
#define SSPOV   6
#define WCOL    7

// OPTION register bits
#define PS0     0
#define PS1     1
#define PS2     2
#define PSA     3
#define T0SE    4
#define RTE     4
#define T0CS    5
#define RTS     5
#define INTEDG  6
#define RBPU_   7

// TRISE register bits
#define TRISE0  0
#define TRISE1  1
#define TRISE2  2
#define PSPMODE 4
#define IBOV    5
#define OBF     6
#define IBF     7

// PIE1 register bits
#define TMR1IE  0
#define TMR2IE  1
#define CCP1IE  2
#define SSPIE   3
#define PSPIE   7

// POR register bits
#define POR_    1

// SSPSTAT register bits
#define BF      0
#define UA      1
#define RW_     2
#define S       3
#define P       4
#define DA_     5


// REFIE macro
#define __RETFIE() #asm ( retfie)

#ifdef NOLONG
  #asm
    MACRO PUSH
        MOVWF temp_WREG  ; Goes to 0x20 or 0xA0
        SWAPF STATUS, W
        BCF   STATUS,RP0 ;Bank 0
        MOVWF temp_STATUS ;STATUS
        MOVF  __WImage,W
        MOVWF temp_WImage ;__WImage
        MOVF  FSR, W
        MOVWF temp_FSR    ; FSR
        MOVF  __FSRImage, W
        MOVWF temp_FSRImage ; __FSRImage
    ENDM

    MACRO POP
        BCF   STATUS,RP0
        MOVF  temp_FSRImage, W
        MOVWF __FSRImage
        MOVF  temp_WImage, W
        MOVWF __WImage
        MOVF  temp_FSR, W
        MOVWF FSR
        SWAPF temp_STATUS, W
        MOVWF STATUS        ;RP0 restored
        SWAPF temp_WREG,F
        SWAPF temp_WREG,W  ;W restored
    ENDM
  #endasm
#else
  #asm
    MACRO PUSH
        MOVWF temp_WREG  ; Goes to 0x20 or 0xA0
        SWAPF STATUS, W
        BCF   STATUS,RP0 ;Bank 0
        MOVWF temp_STATUS ;STATUS
        MOVF  __WImage,W
        MOVWF temp_WImage ;__WImage
        MOVF  FSR, W
        MOVWF temp_FSR    ; FSR
        MOVF  __FSRImage, W
        MOVWF temp_FSRImage ; __FSRImage
        MOVF  __longAC_L, W
        MOVWF temp_longAC_L
        MOVF  __longAC_H, W
        MOVWF temp_longAC_H
        MOVF  __longIX_L, W
        MOVWF temp_longIX_L
        MOVF  __longIX_H, W
        MOVWF temp_longIX_H
    ENDM

    MACRO POP
        BCF   STATUS,RP0
        MOVF  temp_longIX_H, W
        MOVWF __longIX_H
        MOVF  temp_longIX_L, W
        MOVWF __longIX
        MOVF  temp_longAC_H, W
        MOVWF __longAC_H
        MOVF  temp_longAC_L, W
        MOVWF __longAC_L
        MOVF  temp_FSRImage, W
        MOVWF __FSRImage
        MOVF  temp_WImage, W
        MOVWF __WImage
        MOVF  temp_FSR, W
        MOVWF FSR
        SWAPF temp_STATUS, W
        MOVWF STATUS       ;RP0 restored
        SWAPF temp_WREG,F
        SWAPF temp_WREG,W  ;W restored
    ENDM
  #endasm
#endif

#define SaveContext     #asm " PUSH"
#define RestoreContext  #asm " POP"

#endif

#pragma option +l;

